package com.clx.clx_map_poi_search

import android.content.Context
import android.util.Log
import com.amap.api.services.core.LatLonPoint
import com.amap.api.services.core.PoiItem
import com.amap.api.services.geocoder.GeocodeResult
import com.amap.api.services.geocoder.GeocodeSearch
import com.amap.api.services.geocoder.RegeocodeQuery
import com.amap.api.services.geocoder.RegeocodeResult
import com.amap.api.services.poisearch.PoiResult
import com.amap.api.services.poisearch.PoiSearch


class ClxMapPoiSearchImpl : PoiSearch.OnPoiSearchListener, IPoiSearch,
    GeocodeSearch.OnGeocodeSearchListener {

    /**
     * 通过关键字搜索poi
     */
    override fun onSearchKeywords(context: Context?, keywords: String?) {
        Log.d(Constants.TAG, "onSearchKeywords: keywords = $keywords")
        val query = PoiSearch.Query(keywords, Constants.SEARCH_CONTENT, "")
        query.pageNum = 1
        query.pageSize = 50
        val poiSearch = PoiSearch(context, query)
        poiSearch.setOnPoiSearchListener(this)
        poiSearch.searchPOIAsyn()
    }

    /**
     * 通过经纬度搜索poi
     */
    override fun onSearchAround(context: Context?, latitude: Double?, longitude: Double?) {
        Log.d(Constants.TAG, "onSearchAround: latitude = $latitude, longitude = $longitude")
        val query = PoiSearch.Query("", Constants.SEARCH_CONTENT, "")
        query.pageNum = 1
        query.pageSize = 50
        val poiSearch = PoiSearch(context, query)
        poiSearch.bound =
            PoiSearch.SearchBound(LatLonPoint(latitude ?: 0.0, longitude ?: 0.0), 1000)
        poiSearch.setOnPoiSearchListener(this)
        poiSearch.searchPOIAsyn()
    }

    /**
     * 根据 关键字、经纬度、城市搜索poi
     */
    override fun onSearchAroundAll(
        context: Context?,
        keywords: String?,
        latitude: Double?,
        longitude: Double?,
        city: String?,
    ) {
        Log.d(
            Constants.TAG,
            "onSearchAroundAll: keywords = $keywords latitude = $latitude, longitude = $longitude city = $city"
        )
        val query = PoiSearch.Query(keywords, Constants.SEARCH_CONTENT, city)
        query.pageNum = 1
        query.pageSize = 50
        val poiSearch = PoiSearch(context, query)
        if (latitude != null && longitude != null) {
            poiSearch.bound =
                PoiSearch.SearchBound(LatLonPoint(latitude ?: 0.0, longitude ?: 0.0), 1000)
        }
        poiSearch.setOnPoiSearchListener(this)
        poiSearch.searchPOIAsyn()
    }

    /**
     * 逆地址编码
     */
    override fun onGeocodeSearch(context: Context?, latitude: Double?, longitude: Double?) {
        Log.d(Constants.TAG, "onGeocodeSearch: latitude = $latitude, longitude = $longitude")
        val geocodeSearch = GeocodeSearch(context)
        geocodeSearch.setOnGeocodeSearchListener(this)
        // 第一个参数表示一个Latlng，第二参数表示范围多少米，第三个参数表示是火系坐标系还是GPS原生坐标系
        val query = RegeocodeQuery(
            LatLonPoint(latitude ?: 0.0, longitude ?: 0.0),
            1000.0f,
            GeocodeSearch.AMAP
        )
        geocodeSearch.getFromLocationAsyn(query);
    }


    /**===============================PoiSearch回调================================**/

    override fun onPoiSearched(poiResult: PoiResult, code: Int) {
        val arguments: MutableMap<String, Any> = HashMap(2)
        Log.d(Constants.TAG, "onPoiSearched: code = $code poiList size ${poiResult.pois.size}")
        val list = Utils.buildSearchResultList(poiResult, code)
        val data: MutableMap<String, Any> = HashMap()
        data["poiList"] = list
        arguments["searchPOIResult"] = data
        Log.d(Constants.TAG, "onPoiSearched: arguments = $arguments")
        listener?.onPoiResult(arguments)
    }

    override fun onPoiItemSearched(poiItem: PoiItem, p1: Int) {
        Log.d(Constants.TAG, "onPoiItemSearched: poiItem = $poiItem")
    }

    /**
     * 逆地址编码回调
     */
    override fun onRegeocodeSearched(result: RegeocodeResult?, code: Int) {
//        val arguments: MutableMap<String, Any> = HashMap(2)
        Log.d(Constants.TAG, "onRegeocodeSearched: result: $result")
        val map = Utils.buildSearchRegeocodeResultList(result, code)
        Log.d(Constants.TAG, "onRegeocodeSearched: map: $map")
        listener?.onPoiResult(map)
    }

    /**
     * 地址编码回调
     */
    override fun onGeocodeSearched(p0: GeocodeResult?, p1: Int) {
        TODO("Not yet implemented")
    }

    /**===============================poi搜索结果listener================================**/

    private var listener: OnPoiSearchedResult? = null

    fun setOnPoiSearchedResult(listener: OnPoiSearchedResult?) {
        this.listener = listener
    }

    interface OnPoiSearchedResult {
        fun onPoiResult(data: Map<String, Any>)
    }

}